@@ -1,7 +1,10 @@
 from typing import Any, Optional
 
+from mixins import ReprMixin
 
-class AnalysisResult:
+
+class AnalysisResult(ReprMixin):
+    repr_cols = ['name', 'location', 'result']
 
     def __init__(self, result: Any = None, name: Optional[str] = None,
                  location: Optional[str] = None):
